unit rkRichEditor;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Graphics, Controls, ComCtrls, ExtCtrls, StdCtrls,
  Dialogs, RichMemo;

type

  { TfrRichEditor }

  TfrRichEditor = class(TFrame)
    cboFont: TComboBox;
    cboSize: TComboBox;
    imgMain: TImageList;
    lblSize: TLabel;
    lblFont: TLabel;
    pnlSize: TPanel;
    pnlFont: TPanel;
    RichEdit: TRichMemo;
    dlgOpen: TOpenDialog;
    dlgSave: TSaveDialog;
    sbMain: TStatusBar;
    tbMain: TToolBar;
    btnBold: TToolButton;
    btnItal: TToolButton;
    btnUndl: TToolButton;
    btnOpen: TToolButton;
    btnSave: TToolButton;
    btnLink: TToolButton;
    procedure btnLinkClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure EditLinkAction(Sender: TObject; ALinkAction: TLinkAction;
      const info: TLinkMouseInfo; LinkStart, LinkLen: Integer);
    procedure StyleBtnsClick(Sender: TObject);
    procedure EditSelectionChange(Sender: TObject);
  private
    FSaved: Boolean;
    FFileName: string;
    FFontParams: TFontParams;
    procedure UpdateToolBar;
    procedure SetStyle(AStyle: TFontStyle);
  public
    constructor Create(TheOwner: TComponent); override;
  end;

implementation

uses LCLIntf, RichMemoUtils;

{$R *.lfm}

function GetFileName(const ADialog: TFileDialog; var AFileName: string): Boolean;
begin
  Result := False;
  with ADialog do
  begin
    if (AFileName > '') then
      FileName := AFileName
    else
      InitialDir := GetCurrentDir;
    Result := Execute;
    if Result then AFileName := FileName;
  end;
end;


{ TfrRichEditor }

constructor TfrRichEditor.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  btnBold.Tag := Ord(fsBold);
  btnItal.Tag := Ord(fsItalic);
  btnUndl.Tag := Ord(fsUnderLine);
  cboFont.Items.Assign(Screen.Fonts);
  {$IFDEF UNIX}
  RichEdit.Cursor := crIBeam;
  {$ENDIF}
end;

procedure TfrRichEditor.btnOpenClick(Sender: TObject);
begin
  if GetFileName(dlgOpen, FFileName) then
    LoadRTFFile(RichEdit, FFileName);
end;

procedure TfrRichEditor.btnSaveClick(Sender: TObject);
begin
  if GetFileName(dlgSave, FFileName) then
    SaveRTFFile(RichEdit, FFileName);
end;

procedure TfrRichEditor.StyleBtnsClick(Sender: TObject);
begin
  SetStyle(TFontStyle(TToolButton(Sender).Tag));
end;

procedure TfrRichEditor.btnLinkClick(Sender: TObject);
begin
  with RichEdit do
    SetLink(SelStart, SelLength, not IsLink(SelStart));
  UpdateToolBar;
end;

procedure TfrRichEditor.EditLinkAction(Sender: TObject; ALinkAction: TLinkAction;
                         const Info: TLinkMouseInfo; LinkStart, LinkLen: Integer);
begin
  if (Info.Button = mbLeft) then
    OpenURL(RichEdit.GetText(LinkStart, LinkLen));
end;

procedure TfrRichEditor.EditSelectionChange(Sender: TObject);
begin
  RichEdit.GetTextAttributes(RichEdit.SelStart, FFontParams);
  UpdateToolBar;
end;

procedure TfrRichEditor.UpdateToolBar;
begin
  cboFont.Caption := FFontParams.Name;
  cboSize.Caption := IntToStr(FFontParams.Size);
  btnBold.Down := (fsBold in FFontParams.Style);
  btnItal.Down := (fsItalic in FFontParams.Style);
  btnLink.Down := RichEdit.IsLink(RichEdit.SelStart);
  btnUndl.Down := not btnLink.Down and (fsUnderline in FFontParams.Style);
end;

procedure TfrRichEditor.SetStyle(AStyle: TFontStyle);
begin
  if (AStyle in FFontParams.Style) then
    FFontParams.Style := FFontParams.Style - [AStyle]
  else
    FFontParams.Style := FFontParams.Style + [AStyle];
  RichEdit.SetTextAttributes(RichEdit.SelStart, RichEdit.SelLength, FFontParams);
  UpdateToolbar;
end;

initialization
  RegisterClass(TfrRichEditor);

end.

